const string mapInfosFilename = "MapInfos.json";
const string trainingDataFilename = "TrainingData.json";
const string goalTimesFilename = "GoalTimes.json";
const string totdListFilename = "TotdList.dat";

class Storage{	
	Json::Value@ mapInfos = Json::Object();
	TrainingData@ trainingData = null;
	Json::Value@ goalTimes = Json::Object();
	string totdList = "";
	int64 latestTimeStamp = -1;
	
	Storage(){	
	}
	
	void LoadAll(){
		LoadMapInfos();
		LoadTrainingData();
		LoadGoalTimes();
		LoadTotdList();
	}
	
	void LoadMapInfos(){
		auto jsonData = Json::FromFile(IO::FromStorageFolder(mapInfosFilename));
		if(jsonData.GetType() != Json::Type::Object) return;
		@mapInfos = jsonData;
		trace("Storage: loaded "+mapInfos.Length+" mapInfos");
	}

	void LoadTrainingData(){
		auto jsonData = Json::FromFile(IO::FromStorageFolder(trainingDataFilename));
		if(jsonData.GetType() != Json::Type::Array) return;
		@trainingData = TrainingData(jsonData);
		trace("Storage: loaded training data with "+trainingData.mapInfos.Length+" mapInfos");
	}

	void LoadGoalTimes(){
		auto jsonData = Json::FromFile(IO::FromStorageFolder(goalTimesFilename));
		if(jsonData.GetType() != Json::Type::Object) return;
		@goalTimes = jsonData;
		trace("Storage: loaded "+goalTimes.Length+" goalTimes");
	}

	void LoadTotdList(){
		string path = IO::FromStorageFolder(totdListFilename);
		if(!IO::FileExists(path)) return;
		IO::File f(path);
		f.Open(IO::FileMode::Read);
		totdList = f.ReadToEnd();
		f.Close();
		latestTimeStamp = Text::ParseInt64(totdList.SubStr(0,totdList.IndexOf("\n")).Split(",")[2]);
		trace("Storage: loaded totdlist, latestTimeStamp = "+latestTimeStamp);
	}
	
	MapInfo@ GetMapInfo(const string&in uid){
		auto a = mapInfos.Get(uid);
		if(a is null || a.GetType() != Json::Type::Object) return null;
		return MapInfo(a);
	}
	
	void AddMapInfo(MapInfo@ mapInfo){
		mapInfos[mapInfo.mapUID] = mapInfo;
		Json::ToFile(IO::FromStorageFolder(mapInfosFilename),mapInfos);
	}
	
	void RefreshTrainingData(){
		trace("Storage: Refreshing training data");
		if(trainingData is null) @trainingData = TrainingData();
		trainingData.RefreshFromReplayFiles();
		Json::ToFile(IO::FromStorageFolder(trainingDataFilename),trainingData);
	}

	int64 LastTrainingDataRefresh 
	{
		get { return IO::FileModifiedTime(IO::FromStorageFolder(trainingDataFilename)); }
	}
	
	void RefreshTotdList(){
		trace("Storage: Refreshing totd list");
		bool firstRefresh = latestTimeStamp == -1 || totdList == "";

		int nMonths = 100;
		if(!firstRefresh)
			nMonths = (Time::Stamp - latestTimeStamp)/(60*60*24*28) + 2;
		string rawList = LiveGet("/api/token/campaign/month?length="+nMonths+"&offset=0");
		string latestUID = "";
		if(!firstRefresh)
			latestUID = totdList.SubStr(0,totdList.IndexOf(",")); 

		string sortedList = ParseDays(rawList, latestUID) ;
		totdList = sortedList + totdList;

		IO::File f(IO::FromStorageFolder(totdListFilename));
		f.Open(IO::FileMode::Write);
		f.Write(totdList);
		f.Close();
	}

	// returns sorted list of totds from rawList (most recent first)
	string ParseDays(const string&in rawList, const string&in stopUID = ""){ 
		bool first = true;
		string list = rawList;
		string sortedList = "";

		while(true){
			int d = list.IndexOf("days");
			if(d == -1)
				return sortedList;

			list = list.SubStr(d+6);
			auto days = Json::Parse(list.SubStr(0,list.IndexOf("]")+1));
			for(int i = days.Length -1 ; i >= 0; i--){
				string mapUid = days[i]["mapUid"];
				if(mapUid != ""){
					if(mapUid == stopUID)
						return sortedList;

					string seasonUid = days[i]["seasonUid"];
					int64 startTimestamp = days[i]["startTimestamp"];
					if(first){
						latestTimeStamp = startTimestamp;
						first = false;
					}
					sortedList += mapUid +"," + seasonUid + "," +startTimestamp + "\n";
				}
			}
		}
		return sortedList;
	}
	
	int GetGoalTime(const string&in uid){
		return int(goalTimes.Get(uid,Json::Value(-1)));
	}
	
	void SetGoalTime(const string&in uid, int time){
		goalTimes[uid] = time;
		Json::ToFile(IO::FromStorageFolder(goalTimesFilename), goalTimes);
	}
}